from __future__ import unicode_literals, absolute_import

from random import choice

from factory import Sequence
from localflavor.us.us_states import US_STATES

from . import primitives


class Sham(Sequence):

    def __init__(self):
        super(Sham, self).__init__(self.value)

    def value(self, n):
        raise NotImplementedError('Subclasses must implement this.')


class Name(Sham):

    def __init__(self, min_words=2, max_words=2):
        self.min_words = min_words
        self.max_words = max_words
        super(Name, self).__init__()

    def value(self, n):
        return primitives.random_capital_words(self.min_words, self.max_words)


class Blob(Sham):

    def __init__(self, min_words=100, max_words=100):
        self.min_words = min_words
        self.max_words = max_words
        super(Blob, self).__init__()

    def value(self, n):
        return primitives.random_words(self.min_words, self.max_words)


class Number(Sham):

    def __init__(self, min=None, max=None):
        self.min = min
        self.max = max
        super(Number, self).__init__()

    def value(self, n):
        if self.min and self.max:
            return primitives.number(self.min, self.max)
        return n


class UnicodeNumber(Number):

    def __init__(self, digits=None, *args, **kwargs):
        self.digits = digits
        super(UnicodeNumber, self).__init__(*args, **kwargs)

    def value(self, n):
        val = unicode(super(UnicodeNumber, self).value(n))
        if self.digits:
            val = val.zfill(self.digits)[:self.digits]
        return val

# TODO: support other phone formats


class Phone(Sham):

    def value(self, n):
        return '-'.join(
            unicode(primitives.number(0, 10 ** i - 1)) for i in (3, 3, 4))


class Url(Sham):

    def value(self, n):
        return primitives.url()


class StreetAddress(Sham):

    def value(self, n):
        suffixes = ('St', 'Ave', 'Ln', 'Blvd', 'Rd')
        return '{0} {1} {2}.'.format(
            primitives.number(1, 10 ** 5),
            primitives.random_capital_words(1, 3),
            choice(suffixes),
        )


class State(Sham):

    def __init__(self, short_name=False):
        self.short_name = short_name
        super(State, self).__init__()

    def value(self, n):
        index = 0 if self.short_name else 1
        return choice([s[index] for s in US_STATES]).decode('utf-8')


class ZipCode(UnicodeNumber):

    def __init__(self):
        super(ZipCode, self).__init__(digits=5, min=1, max=10 ** 5 - 1)



from __future__ import unicode_literals

import string
from random import choice, randint
from urllib import urlencode
from urlparse import urlunsplit

CHARS = (
    string.lowercase.decode('utf-8') +
    ''.join(set(''.join(unichr(i) for i in range(192, 300)).lower()))
)


def word(min=2, max=10, charset=None):
    charset = charset or CHARS
    return ''.join(choice(charset) for i in range(randint(min, max)))


def _words(word_func, num=3, **kwargs):
    return ' '.join(word_func(**kwargs) for i in range(num))


def lowercase_word(*args, **kwargs):
    return word(**kwargs).lower()


def uppercase_word(*args, **kwargs):
    return word(**kwargs).upper()


def capital_word(*args, **kwargs):
    return word(**kwargs).capitalize()


def words(*args, **kwargs):
    return _words(word, **kwargs)


def random_words(min, max, *args, **kwargs):
    num = randint(min, max)
    return words(num=num, *args, **kwargs)


def lowercase_words(*args, **kwargs):
    return _words(lowercase_word, **kwargs)


def random_lowercase_words(min, max, *args, **kwargs):
    num = randint(min, max)
    return lowercase_words(num=num, *args, **kwargs)


def uppercase_words(*args, **kwargs):
    return _words(uppercase_word, **kwargs)


def random_uppercase_words(min, max, *args, **kwargs):
    num = randint(min, max)
    return uppercase_words(num=num, *args, **kwargs)


def capital_words(*args, **kwargs):
    return _words(capital_word, **kwargs)


def random_capital_words(min, max, *args, **kwargs):
    num = randint(min, max)
    return capital_words(num=num, *args, **kwargs)


def number(min, max):
    return randint(min, max)

def domain():
    tld = choice(('com', 'org', 'net', 'edu', 'gov'))
    charset = (
        string.digits +
        string.lowercase +
        string.uppercase
    ).decode('utf-8')
    return '.'.join((
        random_words(1, 4, charset=charset).replace(' ', '.'),
        tld,
    ))

def url():
    scheme = choice(('http', 'https'))
    charset = (
        string.digits +
        string.lowercase +
        string.uppercase
    ).decode('utf-8')
    path = random_words(1, 4, charset=charset).replace(' ', '/')
    query = urlencode(dict(words(
        num=2, charset=charset).split(' ') for i in range(randint(1, 4))))
    fragment = word(charset=charset)
    return urlunsplit((scheme, domain(), path, query, fragment))

def email():
    charset = (
        string.digits +
        string.lowercase +
        string.uppercase
    ).decode('utf-8')
    prefix = random_words(1, 4, charset=charset).replace(' ', '+')
    return '{0}@{1}'.format(prefix, domain())



#!/usr/bin/env python

import os
import sys

import shams

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

packages = [
    'shams',
]

requires = [
    'factory-boy>=2.3.1',
    'django-localflavor==1.0',
]

with open('README.md') as f:
    readme = f.read()
with open('LICENSE') as f:
    license = f.read()

setup(
    name='shams',
    version=shams.__version__,
    description='A Python library for generating fake data',
    long_description=readme,
    author='David Gouldin',
    author_email='david@gould.in',
    url='https://github.com/dgouldin/python-shams',
    packages=packages,
    package_data={'': ['LICENSE']},
    package_dir={'shams': 'shams'},
    include_package_data=True,
    install_requires=requires,
    license=license,
    zip_safe=False,
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
    ),
)
